package com.uoc.chapi.Commons;

import android.os.Handler;
import android.util.Log;
import android.widget.TextView;

import static com.uoc.chapi.Commons.Cronometro.CronometroState.CRONOMETRO_STATE_PAUSE;
import static com.uoc.chapi.Commons.Cronometro.CronometroState.CRONOMETRO_STATE_RUNNING;
import static com.uoc.chapi.Commons.Cronometro.CronometroState.CRONOMETRO_STATE_STOP;

/**
 * Created by Chapi on 10/03/2017.
 */

public class Cronometro implements Runnable
{
    enum CronometroState {
        CRONOMETRO_STATE_STOP,
        CRONOMETRO_STATE_RUNNING,
        CRONOMETRO_STATE_PAUSE,
    }

    // Atributos privados de la clase
    private TextView etiq; // Etiqueta para mostrar la información
    private String nombrecronometro; // Nombre del cronómetro
    private int segundos, minutos, horas; // Segundos, minutos y horas que lleva activo el cronómetro
    private Handler escribirenUI; // Necesario para modificar la UI
    private CronometroState cronometroState; // Para pausar el cronómetro
    private String salida; // Salida formateada de los datos del cronómetro

    private String TAG = "TFM";


    /**
     * Constructor de la clase
     * @param nombre Nombre del cronómetro
     * @param etiqueta Etiqueta para mostrar información
     */
    public Cronometro(String nombre, TextView etiqueta)
    {
        Log.d(TAG, "Cronometro");
        etiq = etiqueta;
        salida = "";
        segundos = 0;
        minutos = 0;
        horas = 0;
        nombrecronometro = nombre;
        escribirenUI = new Handler();
        cronometroState = CRONOMETRO_STATE_STOP;
    }

    @Override
    /**
     * Acción del cronómetro, contar tiempo en segundo plano
     */
    public void run()
    {
        try
        {
            while(true)
            {
                Thread.sleep(1000);
                salida = "";
                if( cronometroState == CRONOMETRO_STATE_RUNNING)
                {
                    segundos++;
                    if(segundos == 60)
                    {
                        segundos = 0;
                        minutos++;
                    }
                    if(minutos == 60)
                    {
                        minutos = 0;
                        horas++;
                    }
                    // Formateo la salida
                    salida += "0";
                    salida += horas;
                    salida += ":";
                    if( minutos <= 9 )
                    {
                        salida += "0";
                    }
                    salida += minutos;
                    salida += ":";
                    if( segundos <= 9 )
                    {
                        salida += "0";
                    }
                    salida += segundos;
                    // Modifico la UI
                    try
                    {
                        escribirenUI.post(new Runnable()
                        {
                            @Override
                            public void run()
                            {
                                etiq.setText(salida);
                            }
                        });
                    }
                    catch (Exception e)
                    {
                        Log.d(TAG, "Error en el cronometro " + nombrecronometro + " al escribir en la UI: " + e.toString());
                    }
                }
            }
        }
        catch (InterruptedException e)
        {
            Log.d(TAG, "Error en el cronometro " + nombrecronometro + ": " + e.toString());
        }
    }

    public void reset()
    {
        cronometroState = CRONOMETRO_STATE_STOP;
        Log.d(TAG, "State " + cronometroState);
    }

    public void start()
    {
        segundos = 0;
        minutos = 0;
        horas = 0;
        cronometroState = CRONOMETRO_STATE_RUNNING;
        Log.d(TAG, "State " + cronometroState);
    }

    public void pause()
    {
        if (cronometroState == CRONOMETRO_STATE_RUNNING) {
            cronometroState = CRONOMETRO_STATE_PAUSE;
        }
        else if (cronometroState == CRONOMETRO_STATE_PAUSE)
        {
            cronometroState = CRONOMETRO_STATE_RUNNING;
        }
        Log.d(TAG, "State " + cronometroState);
    }
}