package com.uoc.chapi.FollowRoute;

import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;
import com.uoc.chapi.Commons.Commons;
import com.uoc.chapi.GPX.GpxEntry;
import com.uoc.chapi.GPX.GpxPoint;
import com.uoc.chapi.GPX.GpxRoute;
import com.uoc.chapi.GPX.GpxXMLParser;
import com.uoc.chapi.Geo.GeoData;
import com.uoc.chapi.Commons.NameRoute;
import com.uoc.chapi.Routes.R;
import com.uoc.chapi.Commons.ScaleBarOverlay;

import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Chapi on 10/02/2017.
 */

public class RouteToFollowMap extends Fragment {

    private  String TAG = "TFM-RouteToFollowMap";

    private ViewGroup rootView;

    static private GoogleMap myMap;
    private MapView mMapView;
    private ScaleBarOverlay mScaleBar;

    private LocationManager mlocManager;
    private RouteToFollowListener mlocListener;

    private boolean firstTime;

    private String nameRoute;

    private static double currentRange;
    private double currentElevation;
    private double maxAltitude;
    private double minAltitude;
    private double steepPos;
    private double steepNeg;

    private double currentPositionLatitud = 0.0;
    private double currentPositionLongitude = 0.0;

    private Polyline route = null;
    private PolylineOptions routeOpts = null;
    List<LatLng> points;
    ArrayList<GpxPoint> gpxPoints;

    private LinearLayout linearLayoutStart;
    private LinearLayout linearLayoutStopPause;
    private FloatingActionButton floatingActionButtonStopPause;
    private Button buttonStart;
    private Button buttonPause;
    private Button buttonStop;

    private boolean stopPauseLock;
    private static boolean isRunning;
    private boolean isTrackFollowing;

    private Marker mMarker;

    private EditText nameRouteToSave;

    double alturaBasura = 150;
    int contadorBasura = 1;

    double currentAverageElevation;
    double oldAverageElevation;
    double averageElevation;
    int countAverageElevation;
    int numMaxPointtoCalculateAltitude;

    private boolean mediaWithinRoute;
    private boolean mediaOutRoute;


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

        Log.d(TAG, "onCreateView");

        rootView = (ViewGroup) inflater.inflate(R.layout.route_to_follow_map, container, false);

        stopPauseLock = true;
        isRunning = false;
        isTrackFollowing = false;
        mediaWithinRoute = false;
        mediaOutRoute = true;

        InitializeButtons();

        // Initialize position values
        firstTime = true;
        currentRange = 0.0;
        currentPositionLatitud = 0.0;
        currentPositionLongitude = 0.0;
        numMaxPointtoCalculateAltitude = 5;

        // Initialize the Location Listener
        mlocManager = (LocationManager) getContext().getSystemService(Context.LOCATION_SERVICE);
        mlocListener = new RouteToFollowListener();
        mlocListener.setRouteToFollowMap(this);

        if (isLocationGPSEnabled()) {
            Log.d(TAG, "onCreateView-GPS PROVIDER");
            mlocManager.requestLocationUpdates(LocationManager.GPS_PROVIDER
                    , GeoData.PERIOD_SECONDS_GPS
                    , GeoData.DISTANCE_GPS
                    , (LocationListener) mlocListener);
        } else if (isLocationNetworkEnabled()) {
            Log.d(TAG, "onCreateView-NETWORK PROVIDER");
            mlocManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER
                    , GeoData.PERIOD_SECONDS_GPS
                    , GeoData.DISTANCE_GPS
                    , (LocationListener) mlocListener);
        }

        // Initialize the map
        mMapView = (MapView) rootView.findViewById(R.id.routeToFollowMapView);
        mMapView.onCreate(savedInstanceState);
        mMapView.setClickable(true);
        mMapView.setLongClickable(true);

        // Create the context menu
        registerForContextMenu(mMapView);

        mMapView.onResume(); // needed to get the map to display immediately

        try {
            MapsInitializer.initialize(getActivity().getApplicationContext());
        } catch (Exception e) {
            e.printStackTrace();
        }

        mMapView.getMapAsync(new OnMapReadyCallback() {
            @Override
            public void onMapReady(GoogleMap mMap) {

                myMap = mMap;

                // Show the zoom buttons
                myMap.getUiSettings().setZoomControlsEnabled(true);

                // Show the compass
                myMap.getUiSettings().setCompassEnabled(true);

                // Show my location button
                myMap.getUiSettings().setMyLocationButtonEnabled(true);

                // For showing a move to my location button
                myMap.setMyLocationEnabled(true);

                myMap.setMapType(GoogleMap.MAP_TYPE_SATELLITE);

                if (NameRoute.GetNameRouteFull() != null) {
                    nameRoute = NameRoute.GetNameRouteFull();
                    AddRoute(nameRoute);
                }

                myMap.setOnCameraChangeListener(new GoogleMap.OnCameraChangeListener() {
                    @Override
                    public void onCameraChange(CameraPosition cameraPosition) {
                        Log.d (TAG, "OnCameraChangeListener");

                        if (mScaleBar != null) {
                            // This provokes that the imageview is painted again
                            if (myMap.getMapType() == GoogleMap.MAP_TYPE_NONE
                                    || myMap.getMapType() == GoogleMap.MAP_TYPE_NORMAL) {
                                mScaleBar.ColorScaleBar(Color.BLACK);
                            }
                            else {
                                mScaleBar.ColorScaleBar(Color.WHITE);
                            }

                            mScaleBar.invalidate();
                        }
                    }
                });

/***********************************************************
 * This method is useful to check the application
 * It can click over the map and get a new location. It simulates the GPS device.

                myMap.setOnMapClickListener(new GoogleMap.OnMapClickListener() {

                    @Override
                    public void onMapClick(LatLng point) {

                        if (contadorBasura % 2 == 0) {
                            alturaBasura = 150;
                        } else {
                            alturaBasura = 10;
                        }
                        contadorBasura++;

                        Log.d(TAG, "onMapClick (" + point.latitude + ", " + point.longitude + ")");

                        setCurrentLocation(point.latitude, point.longitude, alturaBasura);
                    }
                });
******************************************************************/

                myMap.setOnMapLongClickListener(new GoogleMap.OnMapLongClickListener() {
                    @Override
                    public void onMapLongClick(LatLng latLng) {
                        Log.d(TAG, "onMapLongClick");

                        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
                        builder.setTitle(R.string.ChooseMap);
                        builder.setItems(new CharSequence[] {"Ninguno", "Normal", "Satelite"}
                                , new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        setMapType(which);
                                    }
                                });
                        builder.show();
                    }
                });

                RelativeLayout container = (RelativeLayout) rootView.findViewById(R.id.containerViewRoute);
                RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(800, 800);

                mScaleBar = new ScaleBarOverlay(rootView.getContext(), myMap);
                mScaleBar.setLayoutParams(params);
                mScaleBar.ColorScaleBar(Color.WHITE);

                container.addView(mScaleBar);
            }
        });

        return rootView;
    }

    @Override
    public void setMenuVisibility(final boolean visible) {
        super.setMenuVisibility(visible);
        Log.d(TAG, "setMenuVisibility " + NameRoute.GetNameRouteFull());
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.menu_follow_map, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.menuFollowMapDelete:
                return true;
            case R.id.menuFollowMapShare:
                return true;
            default:
                return super.onContextItemSelected(item);
        }
    }

    /**
     * Initialize buttons
     */
    private void InitializeButtons() {
        buttonStart = (Button) rootView.findViewById(R.id.buttonStartFollow);
        buttonPause = (Button) rootView.findViewById(R.id.buttonPauseFollow);
        buttonStop = (Button) rootView.findViewById(R.id.buttonStopFollow);

        nameRouteToSave = new EditText(getContext());

        linearLayoutStart = (LinearLayout) rootView.findViewById(R.id.boxContainerButtonStartFollow);
        linearLayoutStopPause = (LinearLayout) rootView.findViewById(R.id.boxContainerButtonStopFollow);

        floatingActionButtonStopPause = (FloatingActionButton) rootView.findViewById(R.id.newRouteFloatButtonStopStartFollow);

        linearLayoutStart.setVisibility(View.VISIBLE);
        linearLayoutStopPause.setVisibility(View.INVISIBLE);
        buttonStart.setEnabled(true);
        buttonPause.setEnabled(false);
        buttonStop.setEnabled(false);

        buttonStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {

                Log.d(TAG, "onClisk.start");

                AlertDialog.Builder showMenu = new AlertDialog.Builder(getContext());
                showMenu.setMessage(R.string.wantedStart);
                showMenu.setCancelable(false);
                showMenu.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                        isRunning = true;

                        routeOpts = new PolylineOptions()
                                .color(Color.WHITE)
                                .width(5)
                                .geodesic(true);
                        if (route != null) {
                            route.remove();
                        }
                        route = myMap.addPolyline(routeOpts);
                        route.setVisible(true);

                        currentRange = 0.0;
                        currentElevation = 0.0;
                        maxAltitude = 0.0;
                        minAltitude = 0.0;
                        steepPos = 0.0;
                        steepNeg = 0.0;

                        gpxPoints = new ArrayList<GpxPoint>();

                        RouteToFollowData.startCronometro();
                        linearLayoutStart.setVisibility(View.INVISIBLE);
                        linearLayoutStopPause.setVisibility(View.VISIBLE);
                        floatingActionButtonStopPause.setVisibility(View.VISIBLE);

                        setCurrentLocation(currentPositionLatitud, currentPositionLongitude, currentElevation);
                    }
                });
                showMenu.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                    }
                });
                showMenu.show();
            }
        });

        buttonPause.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                Log.d(TAG, "Pause");

                final EditText nameRoute = new EditText(getContext());
                AlertDialog.Builder showMenu = new AlertDialog.Builder(getContext());
                if (isRunning) {
                    showMenu.setMessage(R.string.wantedPause);
                } else {
                    showMenu.setMessage(R.string.wantedRestart);
                }

                showMenu.setCancelable(false);
                showMenu.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                        stopPauseLock = true;
                        isRunning = !isRunning;
                        RouteToFollowData.pauseCronometro();
                        floatingActionButtonStopPause.setImageDrawable(getResources().getDrawable(R.mipmap.ic_lock));
                    }
                });
                showMenu.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                    }
                });
                showMenu.show();
            }
        });

        buttonStop.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                Log.d(TAG, "Stop");

                AlertDialog.Builder showMenu = new AlertDialog.Builder(getContext());
                showMenu.setMessage(R.string.wantedStop);
                showMenu.setCancelable(false);
                showMenu.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                        stopPauseLock = true;
                        isRunning = false;
                        RouteToFollowData.stopCronometro();

                        if (myMap != null) {
                            myMap.setMyLocationEnabled(true);
                            if (mMarker != null) {
                                mMarker.remove();
                                mMarker = null;
                            }
                        }

                        buttonPause.setEnabled(false);
                        buttonStop.setEnabled(false);
                        linearLayoutStart.setVisibility(View.INVISIBLE);
                        linearLayoutStopPause.setVisibility(View.INVISIBLE);
                        floatingActionButtonStopPause.setVisibility(View.INVISIBLE);
                        floatingActionButtonStopPause.setImageDrawable(getResources().getDrawable(R.mipmap.ic_lock));

                        AlertDialog.Builder showSaveroute = new AlertDialog.Builder(getContext());
                        showSaveroute.setMessage(R.string.wantedSave);
                        showSaveroute.setCancelable(false);
                        showSaveroute.setView(nameRouteToSave);
                        showSaveroute.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialogo1, int id) {

                                String nameFileSaved = new Commons().saveGPXFile(getContext(), nameRouteToSave.getText().toString() + ".gpx", gpxPoints);

                                if (nameFileSaved != null) {
                                    Log.d(TAG, nameRouteToSave.getText().toString() + R.string.hassaved);

                                    NameRoute.SetNameRouteShort(nameRouteToSave.getText().toString());
                                    NameRoute.SetNameRouteFull(nameFileSaved);
                                }
                                else {
                                    Log.d(TAG, nameRouteToSave.getText().toString() + R.string.hasnotsaved);
                                }
                            }
                        });
                        showSaveroute.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialogo1, int id) {
                            }
                        });
                        showSaveroute.show();
                    }
                });
                showMenu.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialogo1, int id) {
                    }
                });
                showMenu.show();
            }
        });

        floatingActionButtonStopPause.setVisibility(View.INVISIBLE);
        floatingActionButtonStopPause.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (stopPauseLock) {
                    floatingActionButtonStopPause.setImageDrawable(getResources().getDrawable(R.mipmap.ic_unlock));
                    buttonPause.setEnabled(true);
                    buttonStop.setEnabled(true);
                    stopPauseLock = false;
                } else {
                    floatingActionButtonStopPause.setImageDrawable(getResources().getDrawable(R.mipmap.ic_lock));
                    buttonPause.setEnabled(false);
                    buttonStop.setEnabled(false);
                    stopPauseLock = true;
                }
            }
        });
    }

    /**
     * It is running
     */
    static boolean IsRunning () {
        return isRunning;
    }

    /**
     * Set the new position on the map
     * @param latitude
     * @param longitude
     * @param elevation
     */
    public void setCurrentLocation(double latitude, double longitude, double elevation) {

        if (!isRunning) {

            currentPositionLatitud = latitude;
            currentPositionLongitude = longitude;
            currentElevation = elevation;

            if (myMap != null) {
                if (mMarker == null) {
                    mMarker = myMap.addMarker(new MarkerOptions()
                            .position(new LatLng(latitude, longitude))
                            .flat(true)
                            .anchor((float) 0.5, (float) 0.5)
                            .rotation((float) 0)
                            .icon(BitmapDescriptorFactory.fromResource(R.mipmap.ic_position)));
                } else {
                    mMarker.setPosition(new LatLng(latitude, longitude));
                    mMarker.setRotation(((float) 0));
                }
            }
        }
        else {
            if (myMap != null) {
                myMap.moveCamera(CameraUpdateFactory.newLatLng(new LatLng(latitude, longitude)));

                Log.d(TAG, "setCurrentLocation (" + latitude + ", " + longitude + ")");
                if (firstTime == true) {
                    firstTime = false;
                    currentRange = 0;
                    currentPositionLatitud = latitude;
                    currentPositionLongitude = longitude;
                    minAltitude = elevation;
                    maxAltitude = elevation;
                    steepPos = 0;
                    steepNeg = 0;

                    currentAverageElevation = elevation;
                    averageElevation = elevation;
                    countAverageElevation = 1;

                    LatLng myLatLng = new LatLng(latitude, longitude);
                    List<LatLng> points = route.getPoints();

                    points.add(myLatLng);
                    route.setPoints(points);

                    gpxPoints.add(new GpxPoint(myLatLng.latitude, myLatLng.longitude, currentAverageElevation, new Date()));

                    Log.d(TAG, "setCurrentLocation Elevation("+ elevation
                            + ") AverageElevation(" + currentAverageElevation + ")");
                    RouteToFollowProfile.SetDetailRoute(currentRange, currentAverageElevation);
                    RouteToFollowData.SetData(currentRange, elevation, maxAltitude, minAltitude, steepPos, steepNeg);
                } else {
                    if (routeOpts != null) {
                        LatLng myLatLng = new LatLng(latitude, longitude);
                        List<LatLng> points = route.getPoints();
                        double range;
                        double bearing;

                        range = GeoData.distance(currentPositionLatitud, currentPositionLongitude
                                , latitude, longitude
                                , 'K');

                        bearing = GeoData.bearing(currentPositionLatitud, currentPositionLongitude
                                , latitude, longitude);

                        // Check if the distance is a error value
                        if (range != Double.NaN
                                && bearing != Double.NaN) {

                            double currentRangeAux = currentRange + range;

                            if (currentRangeAux != Double.NaN) {

                                // Create/Update the marker
                                if (mMarker == null) {
                                    mMarker = myMap.addMarker(new MarkerOptions()
                                            .position(new LatLng(latitude, longitude))
                                            .flat(true)
                                            .anchor((float) 0.5, (float) 0.5)
                                            .rotation((float) bearing)
                                            .icon(BitmapDescriptorFactory.fromResource(R.mipmap.ic_position)));
                                } else {
                                    mMarker.setPosition(new LatLng(latitude, longitude));
                                    mMarker.setRotation(((float) bearing));
                                }

                                if (!isTrackFollowing) {
                                    int index = -1;
                                    index = GpxRoute.LookForTheTrack(myLatLng.latitude, myLatLng.longitude);
                                    if (index != -1) {

                                        if (mediaWithinRoute == false) {
                                            mediaWithinRoute = true;
                                            mediaOutRoute = false;
                                            MediaPlayer mediaPlayer = MediaPlayer.create(getContext(), R.raw.audio_yes_message);
                                            mediaPlayer.start();
                                        }

                                        GpxEntry gpxEntry = GpxRoute.GetGpxEntry(index);
                                        Log.d(TAG, "withinRoute range " + gpxEntry.range + " Elevation : " + gpxEntry.elevation);
                                        RouteToFollowProfile.SetCurrentPosition(gpxEntry.range, gpxEntry.elevation);
                                    } else {

                                        if (mediaOutRoute == false) {
                                            mediaWithinRoute = false;
                                            mediaOutRoute = true;
                                            MediaPlayer mediaPlayer = MediaPlayer.create(getContext(), R.raw.audio_no_message);
                                            mediaPlayer.start();
                                        }
                                    }
                                }

                                currentRange += range;
                                currentPositionLatitud = latitude;
                                currentPositionLongitude = longitude;
                                Log.d(TAG, "setCurrentLocation Range(" + currentRange + ")");

                                if (countAverageElevation == numMaxPointtoCalculateAltitude) {
                                    oldAverageElevation = currentAverageElevation;
                                    averageElevation += elevation;
                                    currentAverageElevation = averageElevation / numMaxPointtoCalculateAltitude;
                                    countAverageElevation = 1;
                                    averageElevation = 0;

                                    if (oldAverageElevation < currentAverageElevation) {
                                        steepPos += (currentAverageElevation - oldAverageElevation);
                                    } else if (oldAverageElevation > currentAverageElevation) {
                                        steepNeg += (oldAverageElevation - currentAverageElevation);
                                    }

                                    if (currentAverageElevation > maxAltitude) {
                                        maxAltitude = currentAverageElevation;
                                    }

                                    if (currentAverageElevation < minAltitude) {
                                        minAltitude = currentAverageElevation;
                                    }
                                } else {
                                    averageElevation += elevation;
                                    countAverageElevation++;
                                }

                                points.add(myLatLng);
                                route.setPoints(points);

                                gpxPoints.add(new GpxPoint(myLatLng.latitude, myLatLng.longitude, currentAverageElevation, new Date()));

                                Log.d(TAG, "setCurrentLocation Elevation(" + elevation
                                        + ") AverageElevation(" + currentAverageElevation + ")");
                                RouteToFollowProfile.SetDetailRoute(currentRange, currentAverageElevation);
                                RouteToFollowData.SetData(currentRange, currentAverageElevation, maxAltitude, minAltitude, steepPos, steepNeg);
                            }
                        }
                    }
                }
            }
        }
    }

    static public void setMapType(int mapType) {
        if (myMap != null) {
            myMap.setMapType(mapType);
        }
    }

    // Add a route into the map
    private void AddRoute(String fileName) {

        Log.d(TAG, "AddRoute " + fileName);

        if (myMap != null) {
            int numPoint = 0;

            try {
                Toolbar toolbar = (Toolbar) getActivity().findViewById(R.id.routeToFollowToolbar);
                toolbar.setSubtitle(GpxXMLParser.parseXML(GpxRoute.GetGpxRoute(), myMap, getActivity(), fileName));
            } catch (XmlPullParserException e) {

                e.printStackTrace();
            } catch (IOException e) {
                Log.d(TAG, "AddRoute Exception : " + e.getMessage().toString());
            }
        }
    }

    // Check if the GPS is enabled
    private boolean isLocationGPSEnabled() {
        return mlocManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
    }

    // Check if the Network is enabled
    private boolean isLocationNetworkEnabled() {
        return mlocManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
    }
}