package com.uoc.chapi.FollowRoute;

import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.jjoe64.graphview.DefaultLabelFormatter;
import com.jjoe64.graphview.GraphView;
import com.jjoe64.graphview.LegendRenderer;
import com.jjoe64.graphview.series.DataPoint;
import com.jjoe64.graphview.series.DataPointInterface;
import com.jjoe64.graphview.series.LineGraphSeries;
import com.jjoe64.graphview.series.OnDataPointTapListener;
import com.jjoe64.graphview.series.PointsGraphSeries;
import com.jjoe64.graphview.series.Series;
import com.uoc.chapi.GPX.GpxEntry;
import com.uoc.chapi.GPX.GpxRoute;
import com.uoc.chapi.Routes.R;

import java.util.ArrayList;

import static com.uoc.chapi.Commons.Commons.NUM_MAX_POINTS;

/**
 * Created by Chapi on 10/02/2017.
 */

public class RouteToFollowProfile extends Fragment {

    private String TAG = "TFM-RouteToFollowProfile";

    private GraphView graph = null;

    static double minAltura = 0.0;
    static double maxAltura = 0.0;

    static double currentPositionX = 0.0;
    static double currentPositionY = 0.0;
    static boolean isWithinRoute = false;

    boolean hasCreatedTheActivity = false;



    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

        Log.d(TAG, "onCreateView");

        ViewGroup viewGroup = (ViewGroup) inflater.inflate(R.layout.route_to_follow_profile, container, false);

        if (graph == null) {
            graph = (GraphView) viewGroup.findViewById(R.id.routeToFollowGraph);
        }

        isWithinRoute = false;

        graph.getGridLabelRenderer().setLabelFormatter(new DefaultLabelFormatter() {
            @Override
            public String formatLabel(double value, boolean isValueX) {
                if (!isValueX) {
                    // show normal x values
                    return super.formatLabel(value, isValueX) + "m";
                } else {
                    // show currency for y values
                    return super.formatLabel(value, isValueX);
                }
            }
        });

        hasCreatedTheActivity = true;

        LoadProfile();

        return viewGroup;
    }

    @Override
    public void setMenuVisibility(final boolean visible) {
        super.setMenuVisibility(visible);
        Log.d(TAG, "setMenuVisibility");

        if (hasCreatedTheActivity) {
            // Load the profile
            LoadProfile();
        }
    }

    void LoadProfile() {

        Log.d(TAG, "LoadProfile");
        ArrayList<GpxEntry> products = GpxRoute.GetGpxRoute();
        LineGraphSeries<DataPoint> series = new LineGraphSeries<DataPoint>();
        series.setColor(Color.argb(100, 0, 0, 127));
        series.setTitle("Km");
        series.setBackgroundColor(Color.argb(25, 0, 0, 127));
        series.setDrawBackground(true);

        Log.d(TAG, "LoadProfile + Size : " + products.size());
        if (products.size() > 0) {
            int offsetPoints;
            int numPointsToDraw;
            int indexOffset = 0;
            int index = 0;
            double range = 0.0;
            double oldX = 0.0;

            if (products.size() > NUM_MAX_POINTS) {
                offsetPoints = products.size() / NUM_MAX_POINTS;
            }
            else {
                offsetPoints = 1;
            }

            numPointsToDraw = products.size() / offsetPoints;
            while (indexOffset < products.size()) {
                if (oldX > products.get(indexOffset).range) {
                    Log.d(TAG, "oldX(" + oldX + ") range(" + products.get(indexOffset).range + ")");
                    products.get(indexOffset).range = oldX;
                } else {
                    oldX = products.get(indexOffset).range;
                }

                series.appendData(new DataPoint(products.get(indexOffset).range, products.get(indexOffset).elevation), true, numPointsToDraw);
                range = products.get(indexOffset).range;
                indexOffset += offsetPoints;
                index++;
            }

            if (index > 0) {
                // set manual X bounds
                graph.getViewport().setMinX(0);
                graph.getViewport().setMaxX(range);
                graph.getViewport().setMinY(minAltura);
                graph.getViewport().setMaxY(maxAltura);
                graph.getViewport().setXAxisBoundsManual(true);
                graph.getViewport().setYAxisBoundsManual(true);

                // enable scaling and scrolling
                graph.getViewport().setScalableY(true);

                graph.getLegendRenderer().setVisible(true);
                graph.getLegendRenderer().setAlign(LegendRenderer.LegendAlign.BOTTOM);

                series.setOnDataPointTapListener(new OnDataPointTapListener() {
                    @Override
                    public void onTap(Series series, DataPointInterface dataPoint) {
                        Toast.makeText(getActivity()
                                , String.format("%.2f", dataPoint.getX()) + "km - " + dataPoint.getY() + "m"
                                , Toast.LENGTH_LONG).show();
                    }
                });

                graph.removeAllSeries();

                if (isWithinRoute) {
                    PointsGraphSeries<DataPoint> seriesPosition = new PointsGraphSeries<>();
                    seriesPosition.appendData(new DataPoint(currentPositionX, currentPositionY), true, 1);
                    seriesPosition.setTitle("Pos");
                    seriesPosition.setSize(10);
                    seriesPosition.setColor(Color.RED);
                    seriesPosition.setShape(PointsGraphSeries.Shape.POINT);
                    graph.addSeries(seriesPosition);
                }

                graph.addSeries(series);
            }
        }
    }

    static public void SetDetailRoute(double aMaxAltura, double aMinAltura) {

        minAltura = aMinAltura;
        maxAltura = aMaxAltura;
    }

    static public void SetCurrentPosition(double aPosX, double aPosY) {

        isWithinRoute = true;
        currentPositionX = aPosX;
        currentPositionY = aPosY;
    }

    static public void SetWithinRoute(boolean value) {

        isWithinRoute = value;
    }
}

