package com.uoc.chapi.GPX;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;

/**
 * Created by Chapi on 15/02/2017.
 */

public class GpxEntry {

    private static final String ns = null;

    public double latitude;
    public double longitude;
    public double elevation;
    public double range;

    public GpxEntry(){
        latitude = longitude = elevation = range = 0.0;
    }

    public GpxEntry(double latitude, double longitude, double elevation, double range) {
        this.latitude = latitude;
        this.longitude = longitude;
        this.elevation = elevation;
        this.range = range;
    }

    // Parses the contents of an entry. If it encounters a title, summary, or link tag, hands them off
    // to their respective "read" methods for processing. Otherwise, skips the tag.
    private GpxEntry readEntry(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, ns, "entry");
        double latitude = 0.0;
        double longitude = 0.0;
        double elevation = 0.0;
        double range = 0.0;
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String name = parser.getName();
            if (name.equals("lat")) {
                latitude = readLatitude(parser);
            } else if (name.equals("lon")) {
                longitude = readLongitude(parser);
            } else if (name.equals("ele")) {
                elevation = readElevation(parser);
            } else {
                skip(parser);
            }
        }
        return new GpxEntry(latitude, longitude, elevation, range);
    }

    // Processes title tags in the feed.
    private double readLatitude(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "lat");
        double latitude = Double.parseDouble(readText(parser));
        parser.require(XmlPullParser.END_TAG, ns, "lat");
        return latitude;
    }

    // Processes title tags in the feed.
    private double readLongitude(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "lon");
        double longitude = Double.parseDouble(readText(parser));
        parser.require(XmlPullParser.END_TAG, ns, "lon");
        return longitude;
    }

    // Processes title tags in the feed.
    private double readElevation(XmlPullParser parser) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, "ele");
        double elevation = Double.parseDouble(readText(parser));
        parser.require(XmlPullParser.END_TAG, ns, "ele");
        return elevation;
    }

    // For the tags title and summary, extracts their text values.
    private String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
        String result = "";
        if (parser.next() == XmlPullParser.TEXT) {
            result = parser.getText();
            parser.nextTag();
        }
        return result;
    }

    private void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
        if (parser.getEventType() != XmlPullParser.START_TAG) {
            throw new IllegalStateException();
        }
        int depth = 1;
        while (depth != 0) {
            switch (parser.next()) {
                case XmlPullParser.END_TAG:
                    depth--;
                    break;
                case XmlPullParser.START_TAG:
                    depth++;
                    break;
            }
        }
    }
}


