package com.uoc.chapi.GPX;

import android.graphics.Color;
import android.support.v4.app.FragmentActivity;
import android.util.DisplayMetrics;
import android.util.Log;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.uoc.chapi.Commons.Commons;
import com.uoc.chapi.FollowRoute.RouteToFollowData;
import com.uoc.chapi.Geo.GeoData;
import com.uoc.chapi.Commons.InfoRouteJSon;
import com.uoc.chapi.FollowRoute.RouteToFollowProfile;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;

/**
 * Created by Chapi on 16/02/2017.
 */

public class GpxXMLParser {

    private static String TAG = "TFM-GpxXMLParser";

    static public String parseXML(ArrayList<GpxEntry> products
            , GoogleMap myMap
            , FragmentActivity fragmentActivity
            , String nameFile) throws XmlPullParserException,IOException {

        String result = "";
        XmlPullParserFactory pullParserFactory;

        pullParserFactory = XmlPullParserFactory.newInstance();
        XmlPullParser parser = pullParserFactory.newPullParser();

        InputStream in_s = new FileInputStream(nameFile);
        parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
        parser.setInput(in_s, null);

        int eventType = parser.getEventType();
        GpxEntry currentProduct = null;

        String trkpt = new String("trkpt");
        String trkptEle = new String("ele");
        String trkptTime = new String("time");

        PolylineOptions route = new PolylineOptions()
                .color(Color.RED)
                .width(10)
                .geodesic(true);

        // This point define the rectangle of the route
        double latitudeMin = 90.0;
        double latitudeMax = -90.0;
        double longitudeMin = 180.0;
        double longitudeMax = -180.0;

        double oldLatitud = 0.0;
        double oldLongitud = 0.0;
        double minAltitude = 10000.0;
        double maxAltitude = -10000.0;

        double oldElevation = 0.0;
        double elevationPos = 0.0;
        double elevationNeg = 0.0;

        Date initTime = new Date();
        Date endTime = new Date();
        String strTime;

        double range = 0.0;

        String name = null;

        boolean firstPoint = true;

        try {
            if (!GpxRoute.GetLoadedRoute()) {
                Log.d(TAG, "parseXML-INI");

                while (eventType != XmlPullParser.END_DOCUMENT
                        && firstPoint == true) {
                    switch (eventType) {
                        case XmlPullParser.START_DOCUMENT:
                            break;

                        case XmlPullParser.START_TAG:
                            name = parser.getName();
                            if (name.equals(trkpt)) {
                                currentProduct = new GpxEntry();
                                currentProduct.latitude = Double.parseDouble(parser.getAttributeValue(null, "lat"));
                                currentProduct.longitude = Double.parseDouble(parser.getAttributeValue(null, "lon"));
                                currentProduct.range = 0.0;
                            } else if (currentProduct != null) {
                                if (name.equals(trkptEle)) {
                                    currentProduct.elevation = Double.parseDouble(parser.nextText());
                                } else if (name.equals(trkptTime)) {
                                    strTime = parser.nextText();
                                    initTime = Commons.simpleDateFormat.parse(strTime.replaceAll("Z$", "+0000"));
                                }
                            }
                            break;
                        case XmlPullParser.END_TAG:
                            name = parser.getName();
                            if (name.equalsIgnoreCase(trkpt) && currentProduct != null) {
                                products.add(currentProduct);
                                route.add(new LatLng(currentProduct.latitude, currentProduct.longitude));

                                oldLatitud = currentProduct.latitude;
                                oldLongitud = currentProduct.longitude;
                                oldElevation = currentProduct.elevation;

                                firstPoint = false;
                            }
                    }
                    eventType = parser.next();
                }

                while (eventType != XmlPullParser.END_DOCUMENT) {
                    switch (eventType) {
                        case XmlPullParser.START_DOCUMENT:
                            break;

                        case XmlPullParser.START_TAG:
                            name = parser.getName();
                            if (name.equals(trkpt)) {
                                currentProduct = new GpxEntry();
                                currentProduct.latitude = Double.parseDouble(parser.getAttributeValue(null, "lat"));
                                currentProduct.longitude = Double.parseDouble(parser.getAttributeValue(null, "lon"));
                            } else if (currentProduct != null) {
                                if (name.equals(trkptEle)) {
                                    currentProduct.elevation = Double.parseDouble(parser.nextText());
                                } else if (name.equals(trkptTime)) {
                                    strTime = parser.nextText();
                                    endTime = Commons.simpleDateFormat.parse(strTime.replaceAll("Z$", "+0000"));
                                }
                            }
                            break;
                        case XmlPullParser.END_TAG:
                            name = parser.getName();
                            if (name != null && name.equalsIgnoreCase(trkpt) && currentProduct != null) {

                                range = range + GeoData.distance(oldLatitud, oldLongitud
                                        , currentProduct.latitude, currentProduct.longitude
                                        , 'K');


                                currentProduct.range = range;

                                products.add(currentProduct);

                                route.add(new LatLng(currentProduct.latitude, currentProduct.longitude));

                                if (oldElevation < currentProduct.elevation) {
                                    elevationPos += currentProduct.elevation - oldElevation;
                                } else {
                                    elevationNeg += oldElevation - currentProduct.elevation;
                                }

                                if (minAltitude >= currentProduct.elevation) {
                                    minAltitude = currentProduct.elevation;
                                } else if (maxAltitude <= currentProduct.elevation) {
                                    maxAltitude = currentProduct.elevation;
                                }

                                if (latitudeMin >= oldLatitud) {
                                    latitudeMin = oldLatitud;
                                }

                                if (latitudeMax < oldLatitud) {
                                    latitudeMax = oldLatitud;
                                }

                                if (longitudeMin >= oldLongitud) {
                                    longitudeMin = oldLongitud;
                                }

                                if (longitudeMax < oldLongitud) {
                                    longitudeMax = oldLongitud;
                                }

                                oldLatitud = currentProduct.latitude;
                                oldLongitud = currentProduct.longitude;
                                oldElevation = currentProduct.elevation;
                            }
                    }
                    eventType = parser.next();
                }

                Log.d(TAG, "parseXML-FIN");
            } else {
                products = GpxRoute.GetGpxRoute();

                Log.d(TAG, "loadRoute-INI-points :" + products.size());

                currentProduct = products.get(0);
                route.add(new LatLng(currentProduct.latitude, currentProduct.longitude));
                oldLatitud = currentProduct.latitude;
                oldLongitud = currentProduct.longitude;
                oldElevation = currentProduct.elevation;

                for (int index = 1; index < products.size(); index++) {
                    currentProduct = products.get(index);
                    route.add(new LatLng(currentProduct.latitude, currentProduct.longitude));

                    range = range + GeoData.distance(oldLatitud, oldLongitud
                            , currentProduct.latitude, currentProduct.longitude
                            , 'K');

                    currentProduct.range = range;

                    route.add(new LatLng(currentProduct.latitude, currentProduct.longitude));

                    if (oldElevation < currentProduct.elevation) {
                        elevationPos += currentProduct.elevation - oldElevation;
                    } else {
                        elevationNeg += oldElevation - currentProduct.elevation;
                    }

                    if (minAltitude >= currentProduct.elevation) {
                        minAltitude = currentProduct.elevation;
                    } else if (maxAltitude <= currentProduct.elevation) {
                        maxAltitude = currentProduct.elevation;
                    }

                    if (latitudeMin >= oldLatitud) {
                        latitudeMin = oldLatitud;
                    }

                    if (latitudeMax < oldLatitud) {
                        latitudeMax = oldLatitud;
                    }

                    if (longitudeMin >= oldLongitud) {
                        longitudeMin = oldLongitud;
                    }

                    if (longitudeMax < oldLongitud) {
                        longitudeMax = oldLongitud;
                    }

                    oldLatitud = currentProduct.latitude;
                    oldLongitud = currentProduct.longitude;
                    oldElevation = currentProduct.elevation;
                }
                Log.d(TAG, "loadRoute-FIN");
            }

            DisplayMetrics metrics = new DisplayMetrics();
            fragmentActivity.getWindowManager().getDefaultDisplay().getMetrics(metrics);

            int zoomLevel = GeoData.getBoundsZoomLevel(new LatLng(latitudeMax, longitudeMin)
                    , new LatLng(latitudeMin, longitudeMax)
                    , metrics.widthPixels
                    , metrics.heightPixels) - 1;

            CameraPosition cameraPosition = new CameraPosition.Builder()
                    .target(new LatLng((latitudeMax + latitudeMin) / 2
                            , (longitudeMax + longitudeMin) / 2))
                    .zoom(zoomLevel)
                    .build();
            myMap.moveCamera(CameraUpdateFactory.newCameraPosition(cameraPosition));
            myMap.animateCamera(CameraUpdateFactory.newCameraPosition(cameraPosition));

            myMap.addPolyline(route);

            Log.d(TAG, "parseXML Position(" + (latitudeMax + latitudeMin) / 2 + ", " + (longitudeMax + longitudeMin) / 2 + ")");

            double diffss = (endTime.getTime() - initTime.getTime()) / 1000;
            int hh = (int)(diffss / 3600);
            int mm = (int)((diffss  - (hh * 3600)) / 60);
            int ss = (int)(diffss - hh * 3600 - mm * 60);
            Log.d(TAG, endTime.getTime() + "-" + initTime.getTime());
            Log.d(TAG, "Time : " + hh + ":" + mm + ":" + ss);

            RouteToFollowProfile.SetDetailRoute(maxAltitude, minAltitude);
            RouteToFollowData.SetData(range, 0.0, maxAltitude, minAltitude, elevationPos, elevationNeg);
            RouteToFollowData.SetTime(hh, mm, ss);
            GpxRoute.SetLoadedRoute(true);
        }
        catch (Exception e) {
            Log.d(TAG, "parseXML : " + e.getMessage().toString());
        }

        if (range < 1) {
            result = String.format("%.0f Mt", range * 1000);

        } else {
            result = String.format("%.2f Km", range);
        }

        result += "  " + String.format("D+ %.0f Mt", elevationPos);
        result += "  " + String.format("D- %.0f Mt", elevationNeg);

        return result;
    }

    static public void parseXMLCFG(String nameFile, String nameFileCFG) throws XmlPullParserException,IOException {

        XmlPullParserFactory pullParserFactory;

        pullParserFactory = XmlPullParserFactory.newInstance();
        XmlPullParser parser = pullParserFactory.newPullParser();

        InputStream in_s = new FileInputStream(nameFile);
        parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
        parser.setInput(in_s, null);

        int eventType = parser.getEventType();
        GpxEntry currentProduct = null;

        String trkpt = new String("trkpt");
        String trkptEle = new String("ele");

        // This point define the rectangle of the route
        double latitudeMin = 90.0;
        double latitudeMax = -90.0;
        double longitudeMin = 180.0;
        double longitudeMax = -180.0;

        double oldLatitud = 0.0;
        double oldLongitud = 0.0;
        double minAltitude = 10000.0;
        double maxAltitude = -10000.0;

        double oldElevation = 0.0;
        double elevationPos = 0.0;
        double elevationNeg = 0.0;

        double range = 0.0;

        String name = null;

        boolean firstPoint = true;

        if (!GpxRoute.GetLoadedRoute()) {
            Log.d(TAG, "parseXML-CFG-INI");

            while (eventType != XmlPullParser.END_DOCUMENT
                    && firstPoint == true) {
                switch (eventType) {
                    case XmlPullParser.START_DOCUMENT:
                        break;

                    case XmlPullParser.START_TAG:
                        name = parser.getName();
                        if (name.equals(trkpt)) {
                            currentProduct = new GpxEntry();
                            currentProduct.latitude = Double.parseDouble(parser.getAttributeValue(null, "lat"));
                            currentProduct.longitude = Double.parseDouble(parser.getAttributeValue(null, "lon"));
                            currentProduct.range = 0.0;
                        } else if (currentProduct != null) {
                            if (name.equals(trkptEle)) {
                                currentProduct.elevation = Double.parseDouble(parser.nextText());
                            }
                        }
                        break;
                    case XmlPullParser.END_TAG:
                        name = parser.getName();
                        if (name.equalsIgnoreCase(trkpt) && currentProduct != null) {

                            oldLatitud = currentProduct.latitude;
                            oldLongitud = currentProduct.longitude;
                            oldElevation = currentProduct.elevation;

                            firstPoint = false;
                        }
                }
                eventType = parser.next();
            }

            while (eventType != XmlPullParser.END_DOCUMENT) {
                switch (eventType) {
                    case XmlPullParser.START_DOCUMENT:
                        break;

                    case XmlPullParser.START_TAG:
                        name = parser.getName();
                        if (name.equals(trkpt)) {
                            currentProduct = new GpxEntry();
                            currentProduct.latitude = Double.parseDouble(parser.getAttributeValue(null, "lat"));
                            currentProduct.longitude = Double.parseDouble(parser.getAttributeValue(null, "lon"));
                        } else if (currentProduct != null) {
                            if (name.equals(trkptEle)) {
                                currentProduct.elevation = Double.parseDouble(parser.nextText());
                            }
                        }
                        break;
                    case XmlPullParser.END_TAG:
                        name = parser.getName();
                        if (name.equalsIgnoreCase(trkpt) && currentProduct != null) {

                            range = range + GeoData.distance(oldLatitud, oldLongitud
                                    , currentProduct.latitude, currentProduct.longitude
                                    , 'K');


                            currentProduct.range = range;

                            if (oldElevation < currentProduct.elevation) {
                                elevationPos += currentProduct.elevation - oldElevation;
                            } else {
                                elevationNeg += oldElevation - currentProduct.elevation;
                            }

                            if (minAltitude >= currentProduct.elevation) {
                                minAltitude = currentProduct.elevation;
                            } else if (maxAltitude <= currentProduct.elevation) {
                                maxAltitude = currentProduct.elevation;
                            }

                            if (latitudeMin >= oldLatitud) {
                                latitudeMin = oldLatitud;
                            }

                            if (latitudeMax < oldLatitud) {
                                latitudeMax = oldLatitud;
                            }

                            if (longitudeMin >= oldLongitud) {
                                longitudeMin = oldLongitud;
                            }

                            if (longitudeMax < oldLongitud) {
                                longitudeMax = oldLongitud;
                            }

                            oldLatitud = currentProduct.latitude;
                            oldLongitud = currentProduct.longitude;
                            oldElevation = currentProduct.elevation;
                        }
                }
                eventType = parser.next();

            }

            Gson gson = new Gson();
            InfoRouteJSon infoRouteJSon = new InfoRouteJSon(nameFile
                    , range
                    , minAltitude
                    , maxAltitude
                    , elevationNeg
                    , elevationPos);
            String cadJson = gson.toJson(infoRouteJSon);
            Log.d(TAG, cadJson);

            FileWriter fileWriter = null;
            try {
                fileWriter = new FileWriter(nameFileCFG, true);

                fileWriter.write(cadJson+ "\n");

                fileWriter.close();

                JsonParser parserJson = new JsonParser();
                FileReader fr = new FileReader(nameFileCFG);
                JsonElement datos = parserJson.parse(fr);

            } catch (Exception ex) {
                System.out.println("Mensaje de la excepción: " + ex.getMessage());
            }

            Log.d(TAG, "parseXML-CFG-FIN");
        }
    }
}
