package com.uoc.chapi;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.Toast;

import com.google.common.io.Files;
import com.uoc.chapi.Commons.Commons;
import com.uoc.chapi.Commons.FileChooser;
import com.uoc.chapi.GPX.GpxXMLParser;
import com.uoc.chapi.ListRoutes.ListRoutes;
import com.uoc.chapi.NewRoute.NewRoute;
import com.uoc.chapi.Routes.R;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import static android.Manifest.permission.WRITE_EXTERNAL_STORAGE;

/**
 * Created by Chapi on 23/02/2017.
 */

public class InitialActivity extends AppCompatActivity {

    private  String TAG = "TFM-InitialActivity";
    public static String DIR_ROUTES = "/TFM_Routes";
    private static String FILE_CFG = "Routes.cfg";

    private static final int REQUEST_WRITE_EXTERNAL_STORAGE = 1 ;
    private static final int REQUEST_LOCATION = 2;
    private boolean bRequestWriteExternalstorage = false;
    private boolean bRequestLocation = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.initial_activity);

        Log.d(TAG, "onCreate");

        Toolbar toolbar = (Toolbar) findViewById(R.id.routeToFollowToolbar);
        setSupportActionBar(toolbar);

        // Check pernmisions
        CheckPermissions();

        // Copy a few routes to check the application
        CopySampleRoutes();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,String permissions[], int[] grantResults) {
        Log.d(TAG, "onRequestPermissionsResult " + "Code : " + requestCode);
        switch (requestCode) {
            case REQUEST_WRITE_EXTERNAL_STORAGE: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    bRequestWriteExternalstorage = true;
                } else {

                }
                return;
            }

            case REQUEST_LOCATION: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    bRequestLocation = true;
                } else {

                }
                return;
            }
        }
    }

    /**
     * Check if the app has the permisio of storage and location
     * @return
     */
    private boolean ShowCheckPermission() {

        boolean result = true;

        if (bRequestWriteExternalstorage == false
                && bRequestLocation == false) {
            result = false;
            Toast toast = Toast.makeText(getApplicationContext(), R.string.notlocationstorage, Toast.LENGTH_LONG);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();
        } else if (bRequestWriteExternalstorage == false) {
            result = false;
            Toast toast = Toast.makeText(getApplicationContext(), R.string.notstorage, Toast.LENGTH_LONG);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();

        } else if (bRequestLocation == false) {
            result = false;
            Toast toast = Toast.makeText(getApplicationContext(), R.string.notlocation, Toast.LENGTH_LONG);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();
        }

        if (result == false) {
            CheckPermissions();
        }

        return result;
    }

    /**
     * Check the presmision of STORAGE and LOCATION
     */
    private void CheckPermissions() {
        Log.d(TAG, "CheckPermissions");
        if (ContextCompat.checkSelfPermission(this, WRITE_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {

            Log.d(TAG, "CheckPermissions-Storage");
            ActivityCompat.requestPermissions(this,
                    new String[]{WRITE_EXTERNAL_STORAGE},
                    REQUEST_WRITE_EXTERNAL_STORAGE);
        } else {
            bRequestWriteExternalstorage = true;
        }

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {
            Log.d(TAG, "CheckPermissions-Location");
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                    REQUEST_LOCATION);
        } else {
            bRequestLocation = true;
        }
    }

    // Init the activity for a new route
    public void initRoute(View v) {
        Log.d(TAG, "initRoute");

        if (ShowCheckPermission()) {
            Context context = v.getContext();
            Intent intent = new Intent(context, NewRoute.class);
            context.startActivity(intent);
        }
    }

    // Init the activity of list of routes
    public void listRoute(View v) {
        Log.d(TAG, "listRoute");

        if (ShowCheckPermission()) {
            Context context = v.getContext();
            Intent intent = new Intent(context, ListRoutes.class);
            context.startActivity(intent);
        }
    }

    public void loadRoute(View v) {
        Log.d(TAG, "loadRoute");

        if (ShowCheckPermission()) {
            new FileChooser(this).setFileListener(new FileChooser.FileSelectedListener() {
                @Override
                public void fileSelected(final File file) {
                    Log.d(TAG, file.getAbsolutePath());

                    if (file.toString().contains(".gpx")) {

                        // Check the state of the storage
                        String estado = Environment.getExternalStorageState();
                        if (estado.equals(Environment.MEDIA_MOUNTED)) {

                            File fileDir = Commons.getFileDir(getApplicationContext());
                            try {

                                if (!fileDir.exists()) {
                                    fileDir.mkdirs();
                                }

                                File fileDest = new File(fileDir + "/" + file.getName());
                                File fileCfg = new File(fileDir + "/" + FILE_CFG);

                                if (!fileDest.exists()) {
                                    Files.copy(file, fileDest);
                                    GpxXMLParser.parseXMLCFG(file.getAbsolutePath(), fileCfg.getAbsolutePath());

                                    Toast toast = Toast.makeText(getApplicationContext(), R.string.hasLoaded, Toast.LENGTH_LONG);
                                    toast.setGravity(Gravity.CENTER, 0, 0);
                                    toast.show();
                                } else {
                                    Toast toast = Toast.makeText(getApplicationContext(), R.string.routeExist, Toast.LENGTH_LONG);
                                    toast.setGravity(Gravity.CENTER, 0, 0);
                                    toast.show();
                                }
                            } catch (Exception e) {
                                Log.d("Copy file : ", e.toString());
                                Toast toast = Toast.makeText(getApplicationContext(), R.string.hasNotLoaded, Toast.LENGTH_LONG);
                                toast.setGravity(Gravity.CENTER, 0, 0);
                                toast.show();
                            }
                        }
                    } else {
                        Toast toast = Toast.makeText(getApplicationContext(), R.string.routeNotValid, Toast.LENGTH_LONG);
                        toast.setGravity(Gravity.CENTER, 0, 0);
                        toast.show();
                    }
                }
            }).showDialog();
        }
    }

    /**
     * Copy a few sample route to check the application
     */
    void CopySampleRoutes() {
        Log.d(TAG, "CopySampleRoutes");
        CopySampleRoute(R.raw.samplecamidloblac, "/Sample Cami Di Loblac.gpx");
        CopySampleRoute(R.raw.samplesanfernando, "/Sample San Fernando.gpx");
    }

    /**
     * Copy the route to check the application
     * @param resource
     * @param sampleRoute
     */
    void CopySampleRoute(int resource, String sampleRoute) {

        File destFileDir = Commons.getFileDir(getApplicationContext());
        InputStream inputSample = getResources().openRawResource(resource);
        BufferedReader bufferSample = new BufferedReader(new InputStreamReader(inputSample));

        FileOutputStream outputSample = null;

        try {
            if (!destFileDir.exists()) {
                destFileDir.mkdirs();
            }

            outputSample = new FileOutputStream(destFileDir + sampleRoute);
            String texto = bufferSample.readLine();
            while(texto != null)
            {
                outputSample.write(texto.getBytes());
                texto = bufferSample.readLine();
            }
        } catch (Exception e) {
            Log.d("Copy file : ", e.toString());
            Toast toast = Toast.makeText(getApplicationContext(), R.string.hasNotLoaded, Toast.LENGTH_LONG);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();
        }
        finally
        {
            try {
                if(inputSample != null)
                inputSample.close();

                if(outputSample != null)
                    outputSample.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
