package com.uoc.chapi.ListRoutes;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


import com.uoc.chapi.Commons.Commons;
import com.uoc.chapi.GPX.GpxRoute;
import com.uoc.chapi.Commons.NameRoute;
import com.uoc.chapi.Routes.R;
import com.uoc.chapi.FollowRoute.RouteToFollow;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Chapi on 11/02/2017.
 */

public class ListRoutes extends AppCompatActivity {

    private  String TAG = "TFM-ListRoutes";

    private File fileDir;

    // Views
    ListView listLocalRoutes = null;
    ArrayList<ListRouteRow> listLocalRows = null;

    // Routes
    private static Map<String, String> listRoutesHistory = new HashMap<String, String>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.list_routes);

        Log.d(TAG, "onCreateView");

        Toolbar toolbar = (Toolbar) findViewById(R.id.listRoutesToolbar);
        setSupportActionBar(toolbar);

        listLocalRoutes = (ListView) findViewById(R.id.ListView_Routes);
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume");

        if (listLocalRows == null) {
            listLocalRows = new ArrayList<ListRouteRow>();
        }

        listLocalRows.clear();

        fileDir = Commons.getFileDir(getApplicationContext());

        if (fileDir != null) {
            File[] files = fileDir.listFiles();

            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    File file = files[i];

                    if (!file.isDirectory()) {
                        if (file.getName().contains(".gpx")) {
                            String nameRoute = file.getName().substring(0, file.getName().lastIndexOf("."));
                            if (!listRoutesHistory.containsKey(nameRoute)) {
                                String rangeAndElevation = "";
                                listLocalRows.add(new ListRouteRow(nameRoute, rangeAndElevation));
                                listRoutesHistory.put(nameRoute, rangeAndElevation);
                            } else {
                                listLocalRows.add(new ListRouteRow(nameRoute, ""));
                            }
                        }
                    }
                }

                listLocalRoutes.setAdapter(new Lista_adaptador(getApplicationContext(), R.layout.list_routes_row, listLocalRows) {
                    @Override
                    public void onEntrada(Object entrada, View view) {
                        TextView texto_superior_entrada = (TextView) view.findViewById(R.id.nameRoute);
                        texto_superior_entrada.setText(((ListRouteRow) entrada).getNameRoute());
                    }
                });

                listLocalRoutes.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> pariente, View view, int posicion, long id) {
                        ListRouteRow choosen = (ListRouteRow) pariente.getItemAtPosition(posicion);

                        NameRoute.SetNameRouteShort(choosen.getNameRoute());
                        NameRoute.SetNameRouteFull(fileDir.getAbsoluteFile() + "/" + choosen.getNameRoute() + ".gpx");
                        GpxRoute.SetLoadedRoute(false);
                        GpxRoute.ClearRoute();
                        Log.d(TAG, "Seleccionado: " + NameRoute.GetNameRouteFull());
                        Context context = getApplicationContext();
                        Intent intent = new Intent(context, RouteToFollow.class);
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        intent.putExtra(RouteToFollow.ARG_NAME_ROUTE, choosen.getNameRoute());
                        context.startActivity(intent);
                    }
                });
            }
        }

        if (listLocalRows.size() == 0) {

            Toast toast = Toast.makeText(getApplicationContext(), "No hay rutas", Toast.LENGTH_LONG);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();
        }
    }


    public abstract static class Lista_adaptador extends BaseAdapter {

        private ArrayList<?> entradas;
        private int R_layout_IdView;
        private Context contexto;

        public Lista_adaptador(Context contexto, int R_layout_IdView, ArrayList<?> entradas) {
            super();
            this.contexto = contexto;
            this.entradas = entradas;
            this.R_layout_IdView = R_layout_IdView;
        }

        @Override
        public View getView(int posicion, View view, ViewGroup pariente) {
            if (view == null) {
                LayoutInflater vi = (LayoutInflater) contexto.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                view = vi.inflate(R_layout_IdView, null);
            }
            onEntrada (entradas.get(posicion), view);
            return view;
        }

        @Override
        public int getCount() {
            return entradas.size();
        }

        @Override
        public Object getItem(int posicion) {
            return entradas.get(posicion);
        }

        @Override
        public long getItemId(int posicion) {
            return posicion;
        }

        /** Devuelve cada una de las entradas con cada una de las vistas a la que debe de ser asociada
         * @param entrada La list_routes_row que será la asociada a la view. La list_routes_row es del tipo del paquete/handler
         * @param view View particular que contendrá los datos del paquete/handler
         */
        public abstract void onEntrada (Object entrada, View view);
    }
}
